@@ -18,6 +18,8 @@
 #endif
 
 #include "easel.h"
+#include "esl_mem.h"
+
 #include "esl_alphabet.h"
 
 
@@ -1008,8 +1010,8 @@ esl_abc_dsqlen(const ESL_DSQ *dsq)
  * Synopsis:  Returns the number of residues in a digital seq.
  *
  * Purpose:   Returns the unaligned length of digitized sequence
- *            <dsq>, in residues, not counting any gaps or
- *            missing data symbols. 
+ *            <dsq>, in residues, not counting any gaps, nonresidues,
+ *            or missing data symbols. 
  */
 int64_t
 esl_abc_dsqrlen(const ESL_ALPHABET *abc, const ESL_DSQ *dsq)
@@ -1667,6 +1669,27 @@ esl_abc_EncodeType(char *type)
   else                                     return eslUNKNOWN;
 }
 
+/* Function:  esl_abc_EncodeTypeMem()
+ * Synopsis:  Convert memory chunk to alphabet type code
+ * Incept:    SRE, Thu 02 Aug 2018 
+ *
+ * Purpose:   Same as <esl_abc_EncodeType()>, but for a
+ *            non-NUL terminated memory chunk <type> of
+ *            length <n>.
+ */
+int
+esl_abc_EncodeTypeMem(char *type, int n)
+{
+  if      (esl_memstrcmp_case(type, n, "amino"))  return eslAMINO;
+  else if (esl_memstrcmp_case(type, n, "rna"))    return eslRNA;
+  else if (esl_memstrcmp_case(type, n, "dna"))    return eslDNA;
+  else if (esl_memstrcmp_case(type, n, "coins"))  return eslCOINS;
+  else if (esl_memstrcmp_case(type, n, "dice"))   return eslDICE;
+  else if (esl_memstrcmp_case(type, n, "custom")) return eslNONSTANDARD;
+  else                                            return eslUNKNOWN;
+}
+
+
 /* Function:  esl_abc_DecodeType()
  * Synopsis:  Returns descriptive string for alphabet type code.
  *
