@@ -1,12 +1,6 @@
-/* str*()-like functions for raw memory "lines" (non-NUL terminated strings).
+/* str*()-like functions for raw char arrays (non-NUL-terminated strings).
  * 
- * Especially when we're parsing input in an ESL_BUFFER, we need to be
- * able to parse non-writable raw memory buffers. Because an
- * ESL_BUFFER may have memory mapped a file, we cannot assume that the
- * input is NUL-terminated, and we can't even assume it's writable (so
- * we can't NUL-terminate it ourselves). These functions provide
- * a set of utilities for parsing raw memory (in terms of a char * pointer
- * and a length in bytes; <char *p>, <esl_pos_t n> by convention).
+ * esl_mem.md has additional notes.
  * 
  * Contents:
  *    1. The esl_mem*() API.
@@ -15,6 +9,8 @@
  */
 #include "esl_config.h"
 
+#include <stdint.h>
+#include <limits.h>
 #include <string.h>
 #include <ctype.h>
 
@@ -76,11 +72,13 @@
  *            <eslEFORMAT> if no valid integer digits are found. Now
  *            <*opt_val> and <*opt_nc> are 0.
  *            
- *            <eslERANGE> on an overflow error. In this case
+ *            <eslERANGE> on overflow or underflow error. Now
  *            <*opt_val> is <INT32_MAX> or <INT32_MIN> for an
  *            overflow or underflow, respectively. <*opt_nc> is
  *            set to the number of characters parsed INCLUDING
- *            the digit that caused the overflow.
+ *            the digit that caused the overflow. There might
+ *            be even more digits following, so parsers need to
+ *            be careful before blithely advancing by <nc>.
  *
  * Throws:    <eslEINVAL> if <base> isn't in range <0..36>. Now
  *            <*opt_nc> and <*opt_val> are 0.
@@ -99,10 +97,16 @@
  *            
  *            I could copy <p> to a temporary allocated string that I
  *            NUL-terminate, then use strtol() or suchlike, but that's
- *            just as awful as what I've done here (rewriting
- *            strtol()). Plus, here I get complete control of the integer
- *            type (<int32_t>) whereas strtol() gives me the less satisfying
- *            <long>.
+ *            even more horrible than what I've done here (rewriting
+ *            strtol()). Plus, here I get complete control of the
+ *            integer type (<int32_t>) whereas strtol() gives me the
+ *            less satisfying <long>.
+ *            
+ *            esl_mem_strtoi32(), _strtoi64(), and _strtoi() repeat
+ *            near-identical code. If you change one, change them all.
+ *            
+ *            Stripped-down version of same code is used in 
+ *            esl_json_ReadInt().
  */
 int
 esl_mem_strtoi32(char *p, esl_pos_t n, int base, int *opt_nc, int32_t *opt_val)
@@ -120,11 +124,11 @@ esl_mem_strtoi32(char *p, esl_pos_t n, int base, int *opt_nc, int32_t *opt_val)
   if      ((base == 0 || base == 16) && i < n-1 && p[i] == '0' && p[i+1] == 'x') 
     { i += 2; base = 16; }
   else if (base == 0 && i < n && p[i] == '0')                                    
-    { i += 1; base = 8; }
+    { i += 1; base = 8; ndigits++; }  // ndigits++ because "0" by itself is 0: weirdly, "0" is an octal literal.
   else if (base == 0) 
     { base = 10; }
 
-  for (ndigits = 0; i < n; i++, ndigits++)
+  for (; i < n; i++, ndigits++)
     {
       if      (isdigit(p[i])) digit = p[i] - '0';
       else if (isupper(p[i])) digit = 10 + (p[i] - 'A');
@@ -153,23 +157,146 @@ esl_mem_strtoi32(char *p, esl_pos_t n, int base, int *opt_nc, int32_t *opt_val)
 	  currval = currval * base - digit;
 	}
     }
+
   if (opt_nc)  { *opt_nc  = (ndigits ? i : 0); }
   if (opt_val) { *opt_val = currval; }
   return (ndigits ? eslOK : eslEFORMAT);
 }
 
+
+/* Function:  esl_mem_strtoi64()
+ * Synopsis:  Convert a chunk of memory to an int64_t.
+ * Incept:    SRE, Thu 02 Aug 2018 [Slaid Cleaves, Drunken Barber's Hand]
+ *
+ * Purpose:   Same as <esl_mem_strtoi32()> but converts to <int64_t>.
+ */
+int
+esl_mem_strtoi64(char *p, esl_pos_t n, int base, int *opt_nc, int64_t *opt_val)
+{
+  esl_pos_t i           = 0;
+  int64_t   sign        = 1;
+  int64_t   currval     = 0;
+  int64_t   digit       = 0;
+  int       ndigits     = 0;
+
+  if    (base < 0 || base == 1 || base > 36)  ESL_EXCEPTION(eslEINVAL, "base must be 2..36 or 0");
+  while (i < n && isspace(p[i])) i++; /* skip leading whitespace */
+  if    (i < n && p[i] == '-')   { sign = -1; i++; }
+
+  if      ((base == 0 || base == 16) && i < n-1 && p[i] == '0' && p[i+1] == 'x') 
+    { i += 2; base = 16; }
+  else if (base == 0 && i < n && p[i] == '0')                                    
+    { i += 1; base = 8; ndigits++; }
+  else if (base == 0) 
+    { base = 10; }
+
+  for (; i < n; i++, ndigits++)
+    {
+      if      (isdigit(p[i])) digit = p[i] - '0';
+      else if (isupper(p[i])) digit = 10 + (p[i] - 'A');
+      else if (islower(p[i])) digit = 10 + (p[i] - 'a');
+      else    break;
+      if (digit >= base) break;
+
+      if (sign == 1)
+	{
+	  if (currval > (INT64_MAX - digit) / base) 
+	    { 
+	      if (opt_val) *opt_val = INT64_MAX; 
+	      if (opt_nc)  *opt_nc  = i+1;
+	      return eslERANGE; 
+	    }
+	  currval = currval * base + digit;
+	}
+      else
+	{
+	  if (currval < (INT64_MIN + digit) / base)
+	    { 
+	      if (opt_val) *opt_val = INT64_MIN; 
+	      if (opt_nc)  *opt_nc  = i+1;
+	      return eslERANGE;
+	    }
+	  currval = currval * base - digit;
+	}
+    }
+  if (opt_nc)  { *opt_nc  = (ndigits ? i : 0); }
+  if (opt_val) { *opt_val = currval; }
+  return (ndigits ? eslOK : eslEFORMAT);
+}
+
+/* Function:  esl_mem_strtoi()
+ * Synopsis:  Convert a chunk of memory to an int.
+ * Incept:    SRE, Thu 02 Aug 2018 [Slaid Cleaves, Junkyard]
+ *
+ * Purpose:   Same as <esl_mem_strtoi32()> but converts to generic <int>.
+ */
+int
+esl_mem_strtoi(char *p, esl_pos_t n, int base, int *opt_nc, int *opt_val)
+{
+  esl_pos_t i           = 0;
+  int       sign        = 1;
+  int       currval     = 0;
+  int       digit       = 0;
+  int       ndigits     = 0;
+
+  if    (base < 0 || base == 1 || base > 36)  ESL_EXCEPTION(eslEINVAL, "base must be 2..36 or 0");
+  while (i < n && isspace(p[i])) i++; /* skip leading whitespace */
+  if    (i < n && p[i] == '-')   { sign = -1; i++; }
+
+  if      ((base == 0 || base == 16) && i < n-1 && p[i] == '0' && p[i+1] == 'x') 
+    { i += 2; base = 16; }
+  else if (base == 0 && i < n && p[i] == '0')                                    
+    { i += 1; base = 8; ndigits++; }
+  else if (base == 0) 
+    { base = 10; }
+
+  for (; i < n; i++, ndigits++)
+    {
+      if      (isdigit(p[i])) digit = p[i] - '0';
+      else if (isupper(p[i])) digit = 10 + (p[i] - 'A');
+      else if (islower(p[i])) digit = 10 + (p[i] - 'a');
+      else    break;
+      if (digit >= base) break;
+
+      if (sign == 1)
+	{
+	  if (currval > (INT_MAX - digit) / base) 
+	    { 
+	      if (opt_val) *opt_val = INT_MAX; 
+	      if (opt_nc)  *opt_nc  = i+1;
+	      return eslERANGE; 
+	    }
+	  currval = currval * base + digit;
+	}
+      else
+	{
+	  if (currval < (INT_MIN + digit) / base)
+	    { 
+	      if (opt_val) *opt_val = INT_MIN; 
+	      if (opt_nc)  *opt_nc  = i+1;
+	      return eslERANGE;
+	    }
+	  currval = currval * base - digit;
+	}
+    }
+  if (opt_nc)  { *opt_nc  = (ndigits ? i : 0); }
+  if (opt_val) { *opt_val = currval; }
+  return (ndigits ? eslOK : eslEFORMAT);
+}
+
+
 /* Function:  esl_mem_strtof()
  * Synopsis:  Convert a chunk of memory to a float.
  * Incept:    SRE, Fri Jun  3 10:52:07 2016 [Hamilton]
  *
- * Purpose:   Convert the text starting at <p> to a float, converting no
+ * Purpose:   Convert a prefix of the char array starting at <p> to a float, converting no
  *            more than <n> characters, i.e. the valid length of
  *            non-NUL terminated memory buffer <p>.
  *            
- *            The floating point representation is parsed as:
+ *            The decimal string representation is parsed as:
  *              - leading whitespace is skipped
- *              - an optional sign for the mantissa, +/-
- *              - a mantissa: (at least one digit must be present)
+ *              - an optional sign for the significand, +/-
+ *              - a significand: (at least one digit must be present)
  *                 - an optional string of digits
  *                 - an optional  '.'
  *                 - an optional string of digits
@@ -180,7 +307,14 @@ esl_mem_strtoi32(char *p, esl_pos_t n, int base, int *opt_nc, int32_t *opt_val)
  *            Or, after whitespace and the optional sign, one of the
  *            following special strings (case-insensitive):
  *                "inf", "infinity", "nan"
-
+ *
+ *            Parsing stops at the first character that isn't part of
+ *            a decimal string representation. For example, given "
+ *            42.0xx", 5 characters are parsed (including the skipped
+ *            leading whitespace, for a result of 42.0. A missing
+ *            exponent, as in " 42.0e-" isn't an <eslEFORMAT> error,
+ *            because it's also parsed as 5 chars to 42.0.
+ *
  *            The converted value is optionally returned in
  *            <*opt_val>, and the number of characters parsed (up to
  *            <n>) is optionally returned in <*opt_n>. The caller can
@@ -188,8 +322,19 @@ esl_mem_strtoi32(char *p, esl_pos_t n, int base, int *opt_nc, int32_t *opt_val)
  *            parsed number.
  *            
  *            Only decimal representations are recognized. Compare to
- *            POSIX strtof(), which also allows hexadecimal
- *            representation (when the mantissa leads with 0x or 0X).
+ *            <strtof()>, which also allows hexadecimal representation
+ *            (when the significand leads with 0x or 0X).
+ *            
+ *            If the representation overflows (e.g. "1e999") the
+ *            result is +/-infinity. If it underflows (e.g. "1e-999")
+ *            the result is 0.  These conversions still return
+ *            <eslOK>.
+ *            
+ *            This function incurs a small roundoff error, typically
+ *            around +/-1 ulp. See notes in esl_mem.md for details.
+ *            When an accuracy guarantee is more important than a
+ *            ~three-fold speed difference, use <esl_memtof()>
+ *            instead.
  *
  * Args:      p         - pointer to text buffer to convert
  *            n         - max number of chars to convert in <p>: p[0..n-1] are valid
@@ -198,13 +343,8 @@ esl_mem_strtoi32(char *p, esl_pos_t n, int base, int *opt_nc, int32_t *opt_val)
  *
  * Returns:   <eslOK> on success.
  * 
- *            <eslEFORMAT> if no mantissa digits are found.
+ *            <eslEFORMAT> if no significand digits are found.
  *            Now <*opt_val> is set to 0 and <*opt_nc> is set to 0.
- *
- * Note:      Think this is stupid? Yeah, I agree. But see note on
- *            <esl_mem_strtoi32()> for why this seems necessary, and
- *            why POSIX strtod()/strtol() don't suffice, especially
- *            if we're going to parse mmap'ed() read-only data.
  */
 int
 esl_mem_strtof(char *p, esl_pos_t n, int *opt_nc, float *opt_val)
@@ -261,7 +401,7 @@ esl_mem_strtof(char *p, esl_pos_t n, int *opt_nc, float *opt_val)
 
 	    while (i < n && isdigit(p[i]))
 	      {
-		exp += 10.*exp + (p[i]-'0');
+		exp = 10.*exp + (p[i]-'0');
 		i++;
 		e++;
 	      }
@@ -454,6 +594,34 @@ esl_memstrcmp(const char *p, esl_pos_t n, const char *s)
 }
 
 
+/* Function:  esl_memstrcmp_case()
+ * Synopsis:  Compare memory chunk and string for equality, case-insensitive
+ * Incept:    SRE, Thu 02 Aug 2018 [Gurf Morlix, One More Second]
+ *
+ * Purpose:   Compare line <p> of length <n> to a NUL-terminated string
+ *            <s>, case-insensitively, and return TRUE if they are
+ *            exactly equal: <strlen(s) == n> and <p[0..n-1] ==
+ *            s[0..n-1]>.  Else, return FALSE.
+ *            
+ * Note:      Return convention differs from strcmp(), which may be confusing.
+ *            strcmp()'s convention of -1/0/+1 return is convenient for use
+ *            in sorting, and we should probably sync to that.
+ */
+int
+esl_memstrcmp_case(const char *p, esl_pos_t n, const char *s)
+{
+  esl_pos_t pos;
+
+  if (p == NULL && n == 0 && (s == NULL || s[0] == '\0')) return TRUE;
+  if (!p || !s)                                           return FALSE;
+  for (pos = 0; pos < n && s[pos] != '\0'; pos++)
+    if (toupper(p[pos]) != toupper(s[pos])) return FALSE;
+  if (pos    != n)    return FALSE;
+  if (s[pos] != '\0') return FALSE;
+  return TRUE;
+}
+  
+
 /* Function:  esl_memstrpfx()
  * Synopsis:  Return TRUE if memory line starts with string.
  *
@@ -776,6 +944,8 @@ main(int argc, char **argv)
  *****************************************************************/
 #ifdef eslMEM_TESTDRIVE
 
+#include "esl_random.h"
+
 static void
 utest_mem_strtoi32(void)
 {
@@ -785,24 +955,89 @@ utest_mem_strtoi32(void)
   int     status;
   
   if ( (status = esl_mem_strtoi32("-1234",          5, 10, &nc, &val)) != eslOK      || nc !=  5 || val !=     -1234) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("\t  -1234",      8, 10, &nc, &val)) != eslOK      || nc !=  8 || val !=     -1234) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("1234",           4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=      1234) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("12345",          4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=      1234) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32(" 0xff",          5,  0, &nc, &val)) != eslOK      || nc !=  5 || val !=       255) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32(" 0777",          4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=        63) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("FFGG",           4, 16, &nc, &val)) != eslOK      || nc !=  2 || val !=       255) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("0xffff",         6,  0, &nc, &val)) != eslOK      || nc !=  6 || val !=     65535) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("0xffffff",       8,  0, &nc, &val)) != eslOK      || nc !=  8 || val !=  16777215) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32(" 2147483647",   11,  0, &nc, &val)) != eslOK      || nc != 11 || val != INT32_MAX) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("-2147483648",   11,  0, &nc, &val)) != eslOK      || nc != 11 || val != INT32_MIN) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32(" 2147483648",   11,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MAX) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("-2147483649",   11,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MIN) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32(" 214748364800", 13,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MAX) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("-214748364900", 13,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MIN) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32(" 0x1234",        3, 16, &nc, &val)) != eslEFORMAT || nc !=  0 || val !=         0) esl_fatal(msg);
-  if ( (status = esl_mem_strtoi32("09999999",       7,  0, &nc, &val)) != eslEFORMAT || nc !=  0 || val !=         0) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi32("\t  -1234",      8, 10, &nc, &val)) != eslOK      || nc !=  8 || val !=     -1234) esl_fatal(msg);  // leading whitespace is skipped
+  if ( (status = esl_mem_strtoi32("1234",           4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=      1234) esl_fatal(msg);  
+  if ( (status = esl_mem_strtoi32("12345",          4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=      1234) esl_fatal(msg);  // n=4 is respected
+  if ( (status = esl_mem_strtoi32(" 0xff",          5,  0, &nc, &val)) != eslOK      || nc !=  5 || val !=       255) esl_fatal(msg);  // leading 0x is detected as hexadecimal
+  if ( (status = esl_mem_strtoi32(" 0777",          4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=        63) esl_fatal(msg);  // leading 0 is detected as octal
+  if ( (status = esl_mem_strtoi32("FFGG",           4, 16, &nc, &val)) != eslOK      || nc !=  2 || val !=       255) esl_fatal(msg);  // parse stops at trailing non-integer chars 
+  if ( (status = esl_mem_strtoi32("0xffff",         6,  0, &nc, &val)) != eslOK      || nc !=  6 || val !=     65535) esl_fatal(msg);  
+  if ( (status = esl_mem_strtoi32("0xffffff",       8,  0, &nc, &val)) != eslOK      || nc !=  8 || val !=  16777215) esl_fatal(msg);  
+  if ( (status = esl_mem_strtoi32(" 2147483647",   11,  0, &nc, &val)) != eslOK      || nc != 11 || val != INT32_MAX) esl_fatal(msg);  // largest valid int32
+  if ( (status = esl_mem_strtoi32("-2147483648",   11,  0, &nc, &val)) != eslOK      || nc != 11 || val != INT32_MIN) esl_fatal(msg);  // smallest valid int32
+  if ( (status = esl_mem_strtoi32(" 2147483648",   11,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MAX) esl_fatal(msg);  // overflow
+  if ( (status = esl_mem_strtoi32("-2147483649",   11,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MIN) esl_fatal(msg);  // underflow
+  if ( (status = esl_mem_strtoi32(" 214748364800", 13,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MAX) esl_fatal(msg);  // <nc> is for the digit that overflowed
+  if ( (status = esl_mem_strtoi32("-214748364900", 13,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MIN) esl_fatal(msg);  //   ... or underflowed
+  if ( (status = esl_mem_strtoi32(" 0x1234",        3, 16, &nc, &val)) != eslEFORMAT || nc !=  0 || val !=         0) esl_fatal(msg);  // leading bare "0x" is an error
+  if ( (status = esl_mem_strtoi32("0XX",            3,  0, &nc, &val)) != eslOK      || nc !=  1 || val !=         0) esl_fatal(msg);  // but a leading 0 is 0 (!)
 }
 
+static void
+utest_mem_strtoi64(void)
+{
+  char    msg[] = "esl_mem_strtoi64() unit test failed";
+  int     nc;
+  int64_t val;
+  int     status;
+  
+  if ( (status = esl_mem_strtoi64("-1234",                   5, 10, &nc, &val)) != eslOK      || nc !=  5 || val !=     -1234) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("\t  -1234",               8, 10, &nc, &val)) != eslOK      || nc !=  8 || val !=     -1234) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("1234",                    4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=      1234) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("12345",                   4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=      1234) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64(" 0xff",                   5,  0, &nc, &val)) != eslOK      || nc !=  5 || val !=       255) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64(" 0777",                   4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=        63) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("FFGG",                    4, 16, &nc, &val)) != eslOK      || nc !=  2 || val !=       255) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("0xffff",                  6,  0, &nc, &val)) != eslOK      || nc !=  6 || val !=     65535) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("0xffffff",                8,  0, &nc, &val)) != eslOK      || nc !=  8 || val !=  16777215) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64(" 9223372036854775807",   20,  0, &nc, &val)) != eslOK      || nc != 20 || val != INT64_MAX) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("-9223372036854775808",   20,  0, &nc, &val)) != eslOK      || nc != 20 || val != INT64_MIN) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64(" 9223372036854775808",   20,  0, &nc, &val)) != eslERANGE  || nc != 20 || val != INT64_MAX) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("-9223372036854775809",   20,  0, &nc, &val)) != eslERANGE  || nc != 20 || val != INT64_MIN) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64(" 922337203685477580800", 22,  0, &nc, &val)) != eslERANGE  || nc != 20 || val != INT64_MAX) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("-922337203685477580900", 22,  0, &nc, &val)) != eslERANGE  || nc != 20 || val != INT64_MIN) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64(" 0x1234",                 3, 16, &nc, &val)) != eslEFORMAT || nc !=  0 || val !=         0) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi64("0XX",                     3,  0, &nc, &val)) != eslOK      || nc !=  1 || val !=         0) esl_fatal(msg); 
+}  
+
+
+static void
+utest_mem_strtoi(void)
+{
+  char    msg[] = "esl_mem_strtoi64() unit test failed";
+  int     nc;
+  int     val;
+  int     status;
+  
+  if ( (status = esl_mem_strtoi("-1234",                   5, 10, &nc, &val)) != eslOK      || nc !=  5 || val !=     -1234) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi("\t  -1234",               8, 10, &nc, &val)) != eslOK      || nc !=  8 || val !=     -1234) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi("1234",                    4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=      1234) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi("12345",                   4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=      1234) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi(" 0xff",                   5,  0, &nc, &val)) != eslOK      || nc !=  5 || val !=       255) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi(" 0777",                   4,  0, &nc, &val)) != eslOK      || nc !=  4 || val !=        63) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi("FFGG",                    4, 16, &nc, &val)) != eslOK      || nc !=  2 || val !=       255) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi("0xffff",                  6,  0, &nc, &val)) != eslOK      || nc !=  6 || val !=     65535) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi("0xffffff",                8,  0, &nc, &val)) != eslOK      || nc !=  8 || val !=  16777215) esl_fatal(msg);
+  if (sizeof(int) == 4) {
+    if ( (status = esl_mem_strtoi(" 2147483647",   11,  0, &nc, &val)) != eslOK      || nc != 11 || val != INT32_MAX) esl_fatal(msg);  // largest valid int32
+    if ( (status = esl_mem_strtoi("-2147483648",   11,  0, &nc, &val)) != eslOK      || nc != 11 || val != INT32_MIN) esl_fatal(msg);  // smallest valid int32
+    if ( (status = esl_mem_strtoi(" 2147483648",   11,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MAX) esl_fatal(msg);  // overflow
+    if ( (status = esl_mem_strtoi("-2147483649",   11,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MIN) esl_fatal(msg);  // underflow
+    if ( (status = esl_mem_strtoi(" 214748364800", 13,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MAX) esl_fatal(msg);  // <nc> is for the digit that overflowed
+    if ( (status = esl_mem_strtoi("-214748364900", 13,  0, &nc, &val)) != eslERANGE  || nc != 11 || val != INT32_MIN) esl_fatal(msg);  //   ... or underflowed
+  } else if (sizeof(int) == 8) {
+    if ( (status = esl_mem_strtoi(" 9223372036854775807",   20,  0, &nc, &val)) != eslOK      || nc != 20 || val != INT64_MAX) esl_fatal(msg);
+    if ( (status = esl_mem_strtoi("-9223372036854775808",   20,  0, &nc, &val)) != eslOK      || nc != 20 || val != INT64_MIN) esl_fatal(msg);
+    if ( (status = esl_mem_strtoi(" 9223372036854775808",   20,  0, &nc, &val)) != eslERANGE  || nc != 20 || val != INT64_MAX) esl_fatal(msg);
+    if ( (status = esl_mem_strtoi("-9223372036854775809",   20,  0, &nc, &val)) != eslERANGE  || nc != 20 || val != INT64_MIN) esl_fatal(msg);
+    if ( (status = esl_mem_strtoi(" 922337203685477580800", 22,  0, &nc, &val)) != eslERANGE  || nc != 20 || val != INT64_MAX) esl_fatal(msg);
+    if ( (status = esl_mem_strtoi("-922337203685477580900", 22,  0, &nc, &val)) != eslERANGE  || nc != 20 || val != INT64_MIN) esl_fatal(msg);
+  }
+  if ( (status = esl_mem_strtoi(" 0x1234",                 3, 16, &nc, &val)) != eslEFORMAT || nc !=  0 || val !=         0) esl_fatal(msg);
+  if ( (status = esl_mem_strtoi("0XX",                     3,  0, &nc, &val)) != eslOK      || nc !=  1 || val !=         0) esl_fatal(msg); 
+}  
+
+
 
 static void
 utest_mem_strtof(void)
@@ -852,13 +1087,61 @@ utest_mem_strtof(void)
   if (( status = esl_mem_strtof("iNfXYZ",           6, &nc, &val) ) != eslOK || nc != 3  ||  !isinf(val))                               esl_fatal(msg);
   if (( status = esl_mem_strtof("nAnXYZ",           6, &nc, &val) ) != eslOK || nc != 3  ||  !isnan(val))                               esl_fatal(msg);
 
+  /* terrifying edge cases that expose some flaws in our implementation, even aside from roundoff error */
+  if (( status = esl_mem_strtof("9999999999999999999999999999999999999999e-10",         44, &nc, &val) ) != eslOK || nc != 44  || val != eslINFINITY)   esl_fatal(msg);   // a real strtof() gets this right: it's 1e30
+  if (( status = esl_mem_strtof("-9999999999999999999999999999999999999999e-10",        45, &nc, &val) ) != eslOK || nc != 45  || val != -eslINFINITY)  esl_fatal(msg);   // ... and this should be -1e30
+  if (( status = esl_mem_strtof("0.0000000000000000000000000000000000000000000001e10",  51, &nc, &val) ) != eslOK || nc != 51  || val != 0.0)           esl_fatal(msg);   // ... 1-e36
+  if (( status = esl_mem_strtof("-0.0000000000000000000000000000000000000000000001e10", 52, &nc, &val) ) != eslOK || nc != 52  || val != 0.0)           esl_fatal(msg);   // ... -1-e36
+
+  /* Bad formats correctly detected: */
   if (( status = esl_mem_strtof("XYZXYZ",           6, &nc, &val) ) != eslEFORMAT || nc != 0  || val != 0.0)                            esl_fatal(msg);
   if (( status = esl_mem_strtof("intinity",         8, &nc, &val) ) != eslEFORMAT || nc != 0  || val != 0.0)                            esl_fatal(msg);
-
-
 }
 
 
+/* utest_mem_strtof_error()
+ * 
+ * Compares <esl_mem_strtof()> to <strtof()>. Assumes that <strtof()>
+ * is accurate: that it finds the nearest floating point
+ * representation for the input string (within +/- 1 ulp and correctly
+ * rounded).  Makes sure that error in <esl_mem_strtof()> is tolerable:
+ * that its relative error is no more than 4 ulps away from
+ * <strtof()>, corresponding to a maximum relative error of ~5e-7.
+ * 
+ * There is an error distribution, and this test can stochastically
+ * fail normally. We force a fixed RNG seed unless caller toggles an
+ * <allow_badluck> option to <TRUE>.
+ * 
+ * See esl_mem.md for notes on rationale for rolling our own strtof(),
+ * and its tradeoffs.
+ */
+static void
+utest_mem_strtof_error(ESL_RANDOMNESS *rng, int allow_badluck)
+{
+  char    msg[] = "mem_strtof() error unit test failed";
+  char    s[32];                                    // random generated string representation of a float. Max len of slen+'.'+flen+'e'+"-xx" = 18.
+  typedef union { float f; uint32_t rep; } flunion; // union gives us access to IEEE754 bitwise representation of a float
+  flunion v1;                                       // value deduced by strtof(), which we assume is accurate
+  flunion v2;                                       // value deduced by esl_mem_strtof(), which is fast and not too inaccurate
+  int     ulperr;                                   // error in integer representation of significand
+  int     trial;
+
+  if (! allow_badluck) esl_randomness_Init(rng, 42);
+
+  for (trial = 0; trial < 100000; trial++)          // check 100,000 numbers
+    {
+      esl_rnd_floatstring(rng, s);
+
+      v1.f = strtof(s, NULL);
+      esl_mem_strtof(s, strlen(s), NULL, &(v2.f));
+      ulperr =  (v1.rep & 0x7fffff) - (v2.rep & 0x7fffff);
+
+      if (ulperr > 4)    esl_fatal(msg);  //   4 ulp = up to 4 \epsilon ~ 5e-7 rel error. 
+    }
+}
+  
+
+
 static void
 utest_memtok(void)
 {
@@ -997,7 +1280,9 @@ utest_memstrcontains(void)
 
 static ESL_OPTIONS options[] = {
    /* name  type         default  env   range togs  reqs  incomp  help                docgrp */
-  {"-h",  eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "show help and usage",                            0},
+  { "-h",  eslARG_NONE,    FALSE, NULL, NULL,  NULL,  NULL, NULL, "show help and usage",                     0},
+  { "-s",  eslARG_INT,      "0",  NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",           0 },
+  { "-x",  eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "allow bad luck (stochastic failures)",    0 },
   { 0,0,0,0,0,0,0,0,0,0},
 };
 static char usage[]  = "[-options]";
@@ -1006,15 +1291,29 @@ static char banner[] = "test driver for mem module";
 int
 main(int argc, char **argv)
 {
-  ESL_GETOPTS    *go          = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  ESL_GETOPTS    *go   = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  ESL_RANDOMNESS *rng  = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+  int   allow_badluck  = esl_opt_GetBoolean(go, "-x");  // if a utest can fail just by chance, let it, instead of suppressing
+
+  fprintf(stderr, "## %s\n", argv[0]);
+  fprintf(stderr, "#  rng seed = %" PRIu32 "\n", esl_randomness_GetSeed(rng));
 
   utest_mem_strtoi32();
+  utest_mem_strtoi64();
+  utest_mem_strtoi();
+
   utest_mem_strtof();
   utest_memtok();
   utest_memspn_memcspn();
   utest_memstrcmp_memstrpfx();
   utest_memstrcontains();
 
+  /* tests that can fail stochastically go last, because they reset RNG seed by default */
+  utest_mem_strtof_error(rng, allow_badluck);
+
+  fprintf(stderr, "#  status = ok\n");
+  
+  esl_randomness_Destroy(rng);
   esl_getopts_Destroy(go);
   return 0;
 }
